// Lucas Keyes
// code for the SoundObject class

#include "soundobject.h"
#include "dxmanager.h"
#include <stdio.h>

extern DirectXManagerObject DXManager;


SoundObject::SoundObject() {

	numcopies = 0;

	lpsound_buffer = NULL;

}

SoundObject::~SoundObject() {

	clear();

}

void SoundObject::clear() {

	stop();
	for (int i = 0; i < numcopies; i++) {
		lpsound_buffer[i]->Release();
		lpsound_buffer[i] = NULL;
	}

	delete[] lpsound_buffer;
}

void SoundObject::load(char *filename, int copies) {

	int length;
	BYTE *sound = NULL;

	length = LoadSound(filename, &sound);
	CreateBuffer(length, copies);
		

	LoadBuffer(sound, length);
	CopyBuffer(copies);

	delete[] sound;
}

void SoundObject::play(BOOL looping) {

	int copy = 0;
	DWORD status;

	if (FAILED(lpsound_buffer[copy]->GetStatus(&status)))
		status = DSBSTATUS_PLAYING;
	while ((copy < numcopies) && (status&DSBSTATUS_PLAYING)) {

		if (++copy < numcopies)
			if (FAILED(lpsound_buffer[copy]->GetStatus(&status)))
				status = DSBSTATUS_PLAYING;
	}

	if (copy < numcopies)
		lpsound_buffer[copy]->Play(0,0,looping?DSBPLAY_LOOPING:0);

}

void SoundObject::stop() {

	for (int i = 0; i < numcopies; i++) {
		lpsound_buffer[i]->Stop();
		lpsound_buffer[i]->SetCurrentPosition(0);
	}
}

void SoundObject::pause() {

	for (int i = 0; i < numcopies; i++) {
		lpsound_buffer[i]->Stop();
	}
}

BOOL SoundObject::CopyBuffer(int copies) {

	// make copies of buffer
	BOOL result = TRUE;		// TRUE if everything went ok
	numcopies = copies;

	for (int i = 1; i<copies; i++) // for each copy
		result = result&&
					SUCCEEDED(DXManager.lpdirectsoundobj->DuplicateSoundBuffer(
						*lpsound_buffer, &(lpsound_buffer[i])));

	return result;
}

BOOL SoundObject::LoadBuffer(BYTE *sound, int length) {

	LPVOID w1, w2;	// write points (use 2 for buffer wraparound)
	DWORD l1, l2;	// length of sound to be written to write pointers
	LPDIRECTSOUNDBUFFER buffer = *lpsound_buffer;

	if (length <= 0) return FALSE;	// bail out if wrong length

	if (buffer->Lock(0,length,&w1,&l1,&w2,&l2,0)	// lock down buffer
		== DSERR_BUFFERLOST) {	// if buffer lost
		buffer->Restore();	// restore then try again
		if (FAILED(buffer->Lock(0,length,&w1,&l1,&w2,&l2,0)))
			return FALSE;	// abort if failed the second time
	}

	CopyMemory(w1,sound,l1);	// load first half

	if (w2 != NULL) CopyMemory(w2, sound+l1,l2);	// load second half

	// unlock the buffer and return
	if (FAILED(buffer->Unlock(w1,l1,w2,l2))) return FALSE;

	return TRUE;
}

BOOL SoundObject::CreateBuffer(int length, int copies) {

	DSBUFFERDESC dsbd;
	PCMWAVEFORMAT pcmwf;

	if (length <= 0) return FALSE;	// bail if length info is wrong
	// init pcmwf, wave format descriptor
	memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
	pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
	pcmwf.wf.nChannels = DS_CHANSPERSAMPLE;
	pcmwf.wf.nSamplesPerSec = DS_SAMPLERATE;
	pcmwf.wf.nBlockAlign = 
		DS_CHANSPERSAMPLE * DS_BITSPERSAMPLE/8;
	pcmwf.wf.nAvgBytesPerSec = 
		pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
	pcmwf.wBitsPerSample = DS_BITSPERSAMPLE;

	// init dsbdesc, direct sound buffer descriptor
	memset(&dsbd, 0, sizeof(DSBUFFERDESC));
	dsbd.dwSize = sizeof(DSBUFFERDESC);
	dsbd.dwFlags = DSBCAPS_STATIC;
	dsbd.dwBufferBytes = length;
	dsbd.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;

	// create sound buffers and return success code
	lpsound_buffer = new LPDIRECTSOUNDBUFFER[copies];
	for (int i = 0; i < copies; i++) lpsound_buffer[i] = NULL;
	return SUCCEEDED(DXManager.lpdirectsoundobj->CreateSoundBuffer(&dsbd,
		&lpsound_buffer[0], NULL));
}

int SoundObject::LoadSound(char *filename, BYTE **buffer) {

	HMMIO hmfr;
	MMCKINFO parent, child;
	WAVEFORMATEX wfmtx;
	int size = 0;

	// reclaim memory from buffer
	delete[]*buffer;

	// init parent and child MMCKINFOS
	parent.ckid = (FOURCC)0;
	parent.cksize = 0;
	parent.fccType = (FOURCC) 0;
	parent.dwDataOffset = 0;
	parent.dwFlags = 0;
	child = parent;

	// open sound file
	hmfr = mmioOpen(filename, NULL, MMIO_READ|MMIO_ALLOCBUF);
	if (hmfr == NULL) return NULL;

	// descend into the RIFF
	parent.fccType = mmioFOURCC('W','A','V','E');
	if (mmioDescend(hmfr, &parent, NULL, MMIO_FINDRIFF)) {
		mmioClose(hmfr,0);  return NULL;	// not a wavefile
	}

	// descend into the WAVEfmt
	child.ckid = mmioFOURCC('f','m','t',' ');
	if (mmioDescend(hmfr, &child, &parent, 0)) {
		mmioClose(hmfr,0); return NULL;		// file has no fmt chunk
	}

	// read the WAVEFMT from the wave file
	if(mmioRead(hmfr,(char*)&wfmtx,sizeof(wfmtx))!=sizeof(wfmtx)) {
		mmioClose(hmfr,0); return NULL; // unable to read fmt chunk
	} 

	// check wave format
	if (wfmtx.wFormatTag != WAVE_FORMAT_PCM) { 
		mmioClose(hmfr,0); return NULL;	// WAVE file is not PCM format
	}

	// ascend back to RIFF level
	if (mmioAscend(hmfr, &child, 0)) {
		mmioClose(hmfr,0); return NULL;	// unable to ascend
	}

	// descend to the data chunk
	child.ckid = mmioFOURCC('d','a','t','a');
	if (mmioDescend(hmfr,&child,&parent,MMIO_FINDCHUNK)){
		mmioClose(hmfr,0); return NULL;	// WAVE file has no data chunk
	}


	// grab memory to store sound
	size = child.cksize;
	*buffer = new BYTE[size];
	if (*buffer == NULL) {
		mmioClose(hmfr,0); return NULL;	// out of memory
	}

	// read the wave data
	if ((int) mmioRead(hmfr,(char*)*buffer,size) != size) {
		// data read failed
		mmioClose(hmfr,0);  delete[] *buffer; return NULL;
	}

	// close file and return
	mmioClose(hmfr,0);
	return size;
} 






		







